package com.songhua.system.service.impl;

import com.songhua.common.exception.ServiceException;
import com.songhua.common.utils.DateUtils;
import com.songhua.common.utils.StringUtils;
import com.songhua.system.domain.PzRawData;
import com.songhua.system.domain.PzRawDataVo;
import com.songhua.system.domain.vo.ShhTicketRulesReqVO;
import com.songhua.system.enums.TicketRulesEnum;
import com.songhua.system.mapper.PzRawDataMapper;
import com.songhua.system.service.IPzRawDataService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.FileOutputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 原始数据导入Service业务层处理
 *
 * @author ruoyi
 * @date 2024-11-04
 */
@Slf4j
@Service
public class PzRawDataServiceImpl implements IPzRawDataService {
    @Autowired
    private PzRawDataMapper pzRawDataMapper;


    /**
     * 查询原始数据导入
     *
     * @param id 原始数据导入主键
     * @return 原始数据导入
     */
    @Override
    public PzRawData selectPzRawDataById(Long id) {
        return pzRawDataMapper.selectPzRawDataById(id);
    }

    /**
     * 查询原始数据导入列表
     *
     * @param pzRawData 原始数据导入
     * @return 原始数据导入
     */
    @Override
    public List<PzRawDataVo> selectPzRawDataList(PzRawData pzRawData) {
        return pzRawDataMapper.selectPzRawDataList(pzRawData);
    }

    /**
     * 新增原始数据导入
     *
     * @param pzRawData 原始数据导入
     * @return 结果
     */
    @Override
    public int insertPzRawData(PzRawData pzRawData) {
        pzRawData.setCreateTime(DateUtils.getNowDate());
        return pzRawDataMapper.insertPzRawData(pzRawData);
    }

    @Override
    public int insertPzRawDataVo(PzRawDataVo pzRawDataVo) {
        pzRawDataVo.setCreateTime(DateUtils.getNowDate());
        return pzRawDataMapper.insertPzRawDataVo(pzRawDataVo);
    }

    /**
     * 修改原始数据导入
     *
     * @param pzRawData 原始数据导入
     * @return 结果
     */
    @Override
    public int updatePzRawData(PzRawData pzRawData) {
        return pzRawDataMapper.updatePzRawData(pzRawData);
    }

    @Override
    public int updatePzRawDataVo(PzRawDataVo pzRawDataVo) {
        return pzRawDataMapper.updatePzRawDataVo(pzRawDataVo);
    }

    /**
     * 批量删除原始数据导入
     *
     * @param ids 需要删除的原始数据导入主键
     * @return 结果
     */
    @Override
    public int deletePzRawDataByIds(Long[] ids) {
        return pzRawDataMapper.deletePzRawDataByIds(ids);
    }

    /**
     * 删除原始数据导入信息
     *
     * @param id 原始数据导入主键
     * @return 结果
     */
    @Override
    public int deletePzRawDataById(Long id) {
        return pzRawDataMapper.deletePzRawDataById(id);
    }

    @Override
    public PzRawDataVo checkIfDataExists(String ticketName) {
        PzRawDataVo pzRawDataVo = pzRawDataMapper.checkIfDataExists(ticketName);
        return pzRawDataVo;
    }

    /**
     * 导入用户数据
     *
     * @param userList        用户数据列表
     * @param isUpdateSupport 是否更新支持，如果已存在，则进行更新数据
     * @param operName        操作用户
     * @return 结果
     */
    @Override
    public String importData(List<PzRawDataVo> userList, Boolean isUpdateSupport, String operName) {
        if (StringUtils.isNull(userList) || userList.size() == 0) {
            throw new ServiceException("导入用户数据不能为空！");
        }
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        for (PzRawDataVo data : userList) {
            try {
                // 检查数据是否已存在
                PzRawDataVo isExist = checkIfDataExists(data.getTicketName());
                data.setTicketId(isExist.getId());
                insertPzRawDataVo(data);
                String msg = "【成功】" + data.toString() + " 导入成功：";
                failureMsg.append(msg).append("\n");
            } catch (Exception e) {
                String msg = "【失败】" + data.toString() + " 导入失败：" + e.getMessage();
                failureMsg.append(msg).append("\n");
            }
        }
        return successMsg.toString();
    }

    @Override
    public void export(PzRawData pzRawData) {
        List<ShhTicketRulesReqVO> list = calculateTotalPrices(pzRawDataMapper.selectPzRawDataList(pzRawData));
        // 创建
        Workbook workbook = new XSSFWorkbook();
        Sheet sheet = workbook.createSheet("销售汇总表统计");
        sheet.setColumnWidth(0, 20 * 260);//设置单元格宽度 ，X代表第几列
        Row headerRow = sheet.createRow(0);
        CellStyle style = workbook.createCellStyle(); // 创建单元格样式
        style.setAlignment(HorizontalAlignment.CENTER); // 设置水平居中对齐
        style.setVerticalAlignment(VerticalAlignment.CENTER); // 设置垂直居中
        Cell cell1 = headerRow.createCell(0);
        cell1.setCellValue("票种名");
        cell1.setCellStyle(style);
        sheet.addMergedRegion(new CellRangeAddress(0, 1, 0, 0));

        Cell cell2 = headerRow.createCell(1);
        cell2.setCellValue("总计");
        cell2.setCellStyle(style);
        sheet.addMergedRegion(new CellRangeAddress(0, 0, 1, 2));

        // 创建子标题行
        Row subHeaderRow = sheet.createRow(1);
        Cell sonCell1 = subHeaderRow.createCell(1);
        sonCell1.setCellValue("张数");
        sonCell1.setCellStyle(style);
        Cell sonCell2 = subHeaderRow.createCell(2);
        sonCell2.setCellValue("金额");
        sonCell2.setCellStyle(style);
        // 填充数据
        for (int i = 0; i < list.size(); i++) {
            ShhTicketRulesReqVO data = list.get(i);
            Row dataRow = sheet.createRow(i + 2); // 从第三行开始填充数据
            dataRow.createCell(0).setCellValue(data.getTicketName());
            dataRow.createCell(1).setCellValue(data.getNumberSheets());
            dataRow.createCell(2).setCellValue(String.valueOf(data.getAmount()));
        }
        // 写入
        String desktopPath = Paths.get(System.getProperty("user.home"), "Desktop").toString();
        try (FileOutputStream fileOut = new FileOutputStream(desktopPath + "\\销售汇总表统计" + new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()) + ".xlsx")) {
            workbook.write(fileOut);
            log.info("Excel 文件已成功导出！");
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                //关闭
                workbook.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 统计导出计算总价公式(票种存在多个规则中)
     *
     * @param list
     * @return
     */
    public List<ShhTicketRulesReqVO> calculateTotalPrices(List<PzRawDataVo> list) {
        List<ShhTicketRulesReqVO> returnList = new ArrayList<>();
        Map<String, PzRawDataVo> map = new HashMap<>();
        list.forEach(e -> {
            List<String> keyByValue = TicketRulesEnum.getKeyByValues(e.getTicketId());
            if (!CollectionUtils.isEmpty(keyByValue)) {
                keyByValue.forEach(name -> {
                    if (map.containsKey(name)) {
                        //如果有相同的票种，则累加数量和金额
                        PzRawDataVo pzRawDataVo = map.get(name);
                        pzRawDataVo.setNumber(pzRawDataVo.getNumber() + e.getNumber());
                        pzRawDataVo.setMoney(pzRawDataVo.getMoney() == null ? BigDecimal.ZERO : pzRawDataVo.getMoney().add(e.getMoney()));
                    } else {
                        PzRawDataVo newPzRawDataVo = new PzRawDataVo();
                        newPzRawDataVo.setNumber(e.getNumber());
                        newPzRawDataVo.setMoney(e.getMoney());
                        map.put(name, newPzRawDataVo);
                    }
                });
            }
        });
        map.forEach((k, v) -> returnList.add(new ShhTicketRulesReqVO(k, v.getNumber(), v.getMoney())));
        return returnList;
    }


    /**
     * 统计导出计算总价公式(票种只存在一个规则中)
     *
     * @param list
     * @return
     */
    public List<ShhTicketRulesReqVO> calculateTotalPrice(List<PzRawDataVo> list) {
        List<ShhTicketRulesReqVO> returnList = new ArrayList<>();
        Map<String, PzRawDataVo> map = new HashMap<>();
        list.forEach(e -> {
            String name = TicketRulesEnum.getKeyByValue(e.getTicketId());
            if (map.containsKey(name)) {
                //如果有相同的票种，则累加数量和金额
                PzRawDataVo pzRawDataVo = map.get(name);
                pzRawDataVo.setNumber(pzRawDataVo.getNumber() + e.getNumber());
                pzRawDataVo.setMoney(pzRawDataVo.getMoney() == null ? BigDecimal.ZERO : pzRawDataVo.getMoney().add(e.getMoney()));
            } else {
                map.put(TicketRulesEnum.getKeyByValue(e.getTicketId()), e);
            }
        });
        return returnList;
    }

}
